/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.servlet.util;

import java.util.*;
import org.xquark.xml.xdbc.*;

public class XMLConnectionPool {

  static private XMLConnectionPool instance = new XMLConnectionPool();

  private HashMap urlMap = new HashMap();
  private String defaultUrl = null;
  private String defaultUser = null;
  private String defaultPassword = null;
  
  private XMLConnectionPool() {
    try {
      Class.forName("org.xquark.repository.RepositoryDriver");
    } catch (Exception e) {
      // No driver found. 
    }
    try {
      Class.forName("org.xquark.extractor.ExtractorDriver");
    } catch (Exception e) {
      // No driver found. 
    }
  }

  public static XMLConnectionPool getConnectionPool() {
    return instance;
  }
  
  public static XMLConnectionPool getConnectionPool(String url, String user, String passwd) {
    instance.defaultUrl = url;
    instance.defaultUser = user;
    instance.defaultPassword = passwd;
    return instance;
  }
  
  public synchronized void clearXMLConnections(String url, String user) throws Exception {
    if (url == null || user == null) return;
    String key = url+":"+user;
    List connList = (List)urlMap.remove(key);
    if (connList != null) {
      Iterator it = connList.iterator();
      while (it.hasNext()) {
	XMLConnection conn = (XMLConnection)it.next();
	conn.close();
      }
    }
  }
  
  public synchronized XMLConnection getXMLConnection(String url, String user, String passwd) throws Exception {
    if (url == null || user == null || passwd == null) return null;
    String key = url+":"+user;
    List connList = (List)urlMap.get(key);
    if (connList == null) {
      connList = new ArrayList();
      urlMap.put(key, connList);
    }
    if (connList.size() > 0)
      return (XMLConnection)connList.remove(connList.size()-1);
    else 
      return XMLDriverManager.getConnection(url, user, passwd);
  }

  public synchronized void freeXMLConnection(String url, String user, XMLConnection conn) throws Exception {
    if (url == null || user == null || conn == null) return;
    String key = url+":"+user;
    List connList = (List)urlMap.get(key);
    // This occurs only if the connections have been cleared; 
    if (connList == null) return;
    connList.add(conn);
  }

  public synchronized void clearXMLConnections() throws Exception {
    clearXMLConnections(defaultUrl, defaultUser);
  }

  public synchronized XMLConnection getXMLConnection() throws Exception {
    return getXMLConnection(defaultUrl, defaultUser, defaultPassword);
  }  

  public synchronized void freeXMLConnection(XMLConnection conn) throws Exception {
    freeXMLConnection(defaultUrl, defaultUser, conn);
  }
}

