/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.servlet.util;

import java.io.*;


public class XQueryBeautifier {
  private static final String keywords[] = 
  {
    "FOR ", "for ", 
    "IN ", "in ",
    "LET ", "let ",
    "WHERE ", "where ",
    "RETURN", "return",
    "SORTBY ", "sortby ",
    "AND ", "and ",
    "OR ", "or ",
    "UNION ", "union ",
    "| ", "| ",
    "NAMESPACE ", "namespace ",
    "SOME ", "some ",
    "EVERY ", "every ",
    "SATISFIES ", "satisfies "
  };
  private static final String builtin[] = 
  {
    "collection", "document", "exists", "empty", "not", "avg", "count", "sum", "min", "max", "contains",
    "starts-with", "ends-with", "dateTime", "date", "current-dateTime", "substring", "concat", "distinct-values"
  };

  boolean incomment = false;
	
  public String beautify(String s)
    throws IOException
  {
    BufferedReader  reader = new BufferedReader(new StringReader(s));
    String line = reader.readLine();
    StringBuffer sb = new StringBuffer();
		
    while(line!=null)
      {
	//System.out.println("line="+line);
	if(line.indexOf("&lt;!--") != -1)
	  {
	    if(incomment==true)
	      sb.append(beautifyCommentLine(line));
	    else
	      {
		incomment = true;
		sb.append(getCommentStart());
		sb.append(beautifyCommentLine(line));
		if(line.indexOf("--&gt;") != -1) {
		  incomment = false;
		  sb.append(getCommentEnd());
		}
	      }
	  }
			
	else if(line.indexOf("--&gt;") != -1)
	  {
	    incomment = false;
	    sb.append(beautifyCommentLine(line));
	    sb.append(getCommentEnd());
	  }
			
	else if(incomment==true)
	  {
	    sb.append(beautifyCommentLine(line));
	  }
			
	else 
	  {
	    sb.append(beautifyQueryLine(line));	
	  }
	line = reader.readLine();
      }
    return sb.toString();
  }
	
	
  String beautifyCommentLine(String line)
  {
    return beautifyIndentation(line+"\r\n");
  }
	
	
  String beautifyQueryLine(String line)
  {
    String xquery =  beautifyIndentation(line);
    for(int i=0; i<keywords.length; i++)
      {
	xquery = beautifyKeyword(xquery, keywords[i], "<FONT COLOR=202080><B>"+keywords[i-i%2+1]+"</B></FONT>");	
      }
    for(int i=0; i<builtin.length; i++) {
      xquery = beautifyKeyword(xquery, builtin[i]+"(", "<FONT COLOR=red>"+builtin[i]+"</FONT>"+"(");	
    }
    return xquery +"\r\n";
  }
	
	
  String beautifyIndentation(String line)
  {
    StringBuffer sb = new StringBuffer();
		
    int i=0; 
    while(i<line.length())
      {
	if (line.charAt(i)==' ') sb.append("&nbsp;");	
	else if (line.charAt(i)=='\t') sb.append("&nbsp;&nbsp;&nbsp;&nbsp;");
	else break;
	i++;
      }
    return sb + line.substring(i) + "<br>";
  }
	
  String getCommentStart()
  {
    return "<FONT color=\"#007000\">";
  }
	
	
  String getCommentEnd()
  {
    return "</FONT>";
  }
	
	
  /**
   * Bind variable value. work with multiple occurences of the same variable
   * 
   */
  public String beautifyKeyword(String html, String var, String value)
  {
    String s = html;
    int index = s.indexOf(var);
	
    //System.out.println("var="+var+"  index="+index);
    
    while(index!=-1) { 
      if (index == 0 || s.charAt(index-1) == ' ' || s.charAt(index-1) == ';' || s.charAt(index-1) == '>' || s.charAt(index-1) == '{' || s.charAt(index-1) == '(') { 
	//System.out.println(s.substring(0, index)+value+s.substring(index+var.length(), s.length())+ " INDEX="+(index+value.length()));
	s = s.substring(0, index)+value+s.substring(index+var.length(), s.length());
      }
      index = s.indexOf(var, index+value.length());
    }
    
    return s;
  }
  
  
  public static void main(String s[])
    throws Exception
  {
    StringBuffer sb = new StringBuffer();
    sb.append("select doc($p)\r\n");
    sb.append("from person.* $p\r\n");
    sb.append("where $p.address exists\r\n");
		
    XQueryBeautifier beau = new XQueryBeautifier(); 
    System.out.println("HTML="+beau.beautify(sb.toString()));
  }
}

